@@ -6,11 +6,14 @@
 
 use {
     crate::{
+        rfc3447::DigestInfo,
         rfc5280::{AlgorithmIdentifier, AlgorithmParameter},
         X509CertificateError as Error,
     },
-    bcder::{ConstOid, Oid},
+    bcder::{encode::Values, ConstOid, OctetString, Oid},
     ring::{digest, signature},
+    spki::ObjectIdentifier,
+    std::fmt::{Display, Formatter},
 };
 
 /// SHA-1 digest algorithm.
@@ -126,6 +129,17 @@ pub enum DigestAlgorithm {
     Sha512,
 }
 
+impl Display for DigestAlgorithm {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self {
+            DigestAlgorithm::Sha1 => f.write_str("SHA-1"),
+            DigestAlgorithm::Sha256 => f.write_str("SHA-256"),
+            DigestAlgorithm::Sha384 => f.write_str("SHA-384"),
+            DigestAlgorithm::Sha512 => f.write_str("SHA-512"),
+        }
+    }
+}
+
 impl From<DigestAlgorithm> for Oid {
     fn from(alg: DigestAlgorithm) -> Self {
         Oid(match alg {
@@ -189,6 +203,84 @@ impl DigestAlgorithm {
     pub fn digester(&self) -> digest::Context {
         digest::Context::from(*self)
     }
+
+    /// Digest a slice of data.
+    pub fn digest_data(&self, data: &[u8]) -> Vec<u8> {
+        let mut h = self.digester();
+        h.update(data);
+        h.finish().as_ref().to_vec()
+    }
+
+    /// Digest content from a reader.
+    pub fn digest_reader<R: std::io::Read>(&self, fh: &mut R) -> Result<Vec<u8>, std::io::Error> {
+        let mut h = self.digester();
+
+        loop {
+            let mut buffer = [0u8; 16384];
+            let count = fh.read(&mut buffer)?;
+
+            h.update(&buffer[0..count]);
+
+            if count < buffer.len() {
+                break;
+            }
+        }
+
+        Ok(h.finish().as_ref().to_vec())
+    }
+
+    /// Digest the content of a path.
+    pub fn digest_path(&self, path: &std::path::Path) -> Result<Vec<u8>, std::io::Error> {
+        self.digest_reader(&mut std::fs::File::open(path)?)
+    }
+
+    /// EMSA-PKCS1-v1_5 padding procedure.
+    ///
+    /// As defined by https://tools.ietf.org/html/rfc3447#section-9.2.
+    ///
+    /// `message` is the message to digest and encode.
+    ///
+    /// `target_length_in_bytes` is the target length of the padding. This should match the RSA
+    /// key length. e.g. 2048 bit keys are length 256.
+    pub fn rsa_pkcs1_encode(
+        &self,
+        message: &[u8],
+        target_length_in_bytes: usize,
+    ) -> Result<Vec<u8>, Error> {
+        let digest = self.digest_data(message);
+
+        let digest_info = DigestInfo {
+            algorithm: (*self).into(),
+            digest: OctetString::new(digest.into()),
+        };
+        let mut digest_info_der = vec![];
+        digest_info.write_encoded(bcder::Mode::Der, &mut digest_info_der)?;
+
+        let encoded_digest_len = digest_info_der.len();
+
+        // At least 8 bytes of padding are required. And there's a 2 byte header plus NULL
+        // termination of the padding. So the target length must be 11+ bytes longer than
+        // the encoded digest.
+        if encoded_digest_len + 11 > target_length_in_bytes {
+            return Err(Error::PkcsEncodeTooShort);
+        }
+
+        let pad_len = target_length_in_bytes - encoded_digest_len - 3;
+
+        let mut res = vec![0xff; target_length_in_bytes];
+        // Constant header.
+        res[0] = 0x00;
+        // Private key block type.
+        res[1] = 0x01;
+        // Padding bytes are already filled in.
+        // NULL terminate padding.
+        res[2 + pad_len] = 0x00;
+
+        let digest_destination = &mut res[3 + pad_len..];
+        digest_destination.copy_from_slice(&digest_info_der);
+
+        Ok(res)
+    }
 }
 
 /// An algorithm used to digitally sign content.
@@ -320,6 +412,34 @@ impl SignatureAlgorithm {
             },
         }
     }
+
+    /// Resolve the [DigestAlgorithm] for this signature algorithm.
+    pub fn digest_algorithm(&self) -> Option<DigestAlgorithm> {
+        match self {
+            SignatureAlgorithm::RsaSha1 => Some(DigestAlgorithm::Sha1),
+            SignatureAlgorithm::RsaSha256 => Some(DigestAlgorithm::Sha256),
+            SignatureAlgorithm::RsaSha384 => Some(DigestAlgorithm::Sha384),
+            SignatureAlgorithm::RsaSha512 => Some(DigestAlgorithm::Sha512),
+            SignatureAlgorithm::EcdsaSha256 => Some(DigestAlgorithm::Sha256),
+            SignatureAlgorithm::EcdsaSha384 => Some(DigestAlgorithm::Sha384),
+            // TODO there's got to be a digest algorithm, right?
+            SignatureAlgorithm::Ed25519 => None,
+        }
+    }
+}
+
+impl Display for SignatureAlgorithm {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self {
+            SignatureAlgorithm::RsaSha1 => f.write_str("SHA-1 with RSA encryption"),
+            SignatureAlgorithm::RsaSha256 => f.write_str("SHA-256 with RSA encryption"),
+            SignatureAlgorithm::RsaSha384 => f.write_str("SHA-384 with RSA encryption"),
+            SignatureAlgorithm::RsaSha512 => f.write_str("SHA-512 with RSA encryption"),
+            SignatureAlgorithm::EcdsaSha256 => f.write_str("ECDSA with SHA-256"),
+            SignatureAlgorithm::EcdsaSha384 => f.write_str("ECDSA with SHA-384"),
+            SignatureAlgorithm::Ed25519 => f.write_str("ED25519"),
+        }
+    }
 }
 
 impl From<SignatureAlgorithm> for Oid {
@@ -443,6 +563,16 @@ pub enum KeyAlgorithm {
     Ed25519,
 }
 
+impl Display for KeyAlgorithm {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self {
+            Self::Rsa => f.write_str("RSA"),
+            Self::Ecdsa(_) => f.write_str("ECDSA"),
+            Self::Ed25519 => f.write_str("ED25519"),
+        }
+    }
+}
+
 impl TryFrom<&Oid> for KeyAlgorithm {
     type Error = Error;
 
@@ -462,6 +592,24 @@ impl TryFrom<&Oid> for KeyAlgorithm {
     }
 }
 
+impl TryFrom<&ObjectIdentifier> for KeyAlgorithm {
+    type Error = Error;
+
+    fn try_from(v: &ObjectIdentifier) -> Result<Self, Self::Error> {
+        // Similar implementation as above.
+        match v.as_bytes() {
+            x if x == OID_RSA.as_ref() => Ok(Self::Rsa),
+            x if x == OID_EC_PUBLIC_KEY.as_ref() => Ok(Self::Ecdsa(EcdsaCurve::Secp384r1)),
+            x if x == OID_ED25519_KEY_AGREEMENT.as_ref()
+                || x == OID_ED25519_SIGNATURE_ALGORITHM.as_ref() =>
+            {
+                Ok(Self::Ed25519)
+            }
+            _ => Err(Error::UnknownKeyAlgorithm(v.to_string())),
+        }
+    }
+}
+
 impl From<KeyAlgorithm> for Oid {
     fn from(alg: KeyAlgorithm) -> Self {
         Oid(match alg {
@@ -473,6 +621,18 @@ impl From<KeyAlgorithm> for Oid {
     }
 }
 
+impl From<KeyAlgorithm> for ObjectIdentifier {
+    fn from(alg: KeyAlgorithm) -> Self {
+        let bytes = match alg {
+            KeyAlgorithm::Rsa => OID_RSA.as_ref(),
+            KeyAlgorithm::Ecdsa(_) => OID_EC_PUBLIC_KEY.as_ref(),
+            KeyAlgorithm::Ed25519 => OID_ED25519_KEY_AGREEMENT.as_ref(),
+        };
+
+        ObjectIdentifier::from_bytes(bytes).expect("OID bytes should be valid")
+    }
+}
+
 impl TryFrom<&AlgorithmIdentifier> for KeyAlgorithm {
     type Error = Error;
 
@@ -530,3 +690,44 @@ impl From<KeyAlgorithm> for AlgorithmIdentifier {
         }
     }
 }
+
+#[cfg(test)]
+mod test {
+    use super::*;
+
+    #[test]
+    fn digest_pkcs1() -> Result<(), Error> {
+        let message = b"deadbeef";
+        let raw_digest = DigestAlgorithm::Sha256.digest_data(message);
+
+        // RSA 1024.
+        let encoded = DigestAlgorithm::Sha256.rsa_pkcs1_encode(message, 128)?;
+        assert_eq!(&encoded[0..3], &[0x00, 0x01, 0xff]);
+        assert_eq!(&encoded[96..], &raw_digest);
+
+        Ok(())
+    }
+
+    #[test]
+    fn key_algorithm_oids() -> Result<(), Error> {
+        let oid = ObjectIdentifier::from(KeyAlgorithm::Rsa);
+        assert_eq!(oid.to_string(), "1.2.840.113549.1.1.1");
+        let oid = ObjectIdentifier::new("1.2.840.113549.1.1.1");
+        assert_eq!(KeyAlgorithm::try_from(&oid)?, KeyAlgorithm::Rsa);
+
+        let oid = ObjectIdentifier::from(KeyAlgorithm::Ecdsa(EcdsaCurve::Secp256r1));
+        assert_eq!(oid.to_string(), "1.2.840.10045.2.1");
+        let oid = ObjectIdentifier::new("1.2.840.10045.2.1");
+        assert_eq!(
+            KeyAlgorithm::try_from(&oid)?,
+            KeyAlgorithm::Ecdsa(EcdsaCurve::Secp384r1)
+        );
+
+        let oid = ObjectIdentifier::from(KeyAlgorithm::Ed25519);
+        assert_eq!(oid.to_string(), "1.3.101.110");
+        let oid = ObjectIdentifier::new("1.3.101.110");
+        assert_eq!(KeyAlgorithm::try_from(&oid)?, KeyAlgorithm::Ed25519);
+
+        Ok(())
+    }
+}
