@@ -6,8 +6,9 @@
 
 use {
     crate::{
-        algorithm::DigestAlgorithm, asn1time::Time, rfc3280::Name, rfc5280, InMemorySigningKeyPair,
-        KeyAlgorithm, SignatureAlgorithm, X509CertificateError as Error,
+        algorithm::DigestAlgorithm, asn1time::Time, rfc2986, rfc3280::Name, rfc5280, rfc5652,
+        rfc5958::Attributes, rfc8017::RsaPublicKey, signing::Sign, InMemorySigningKeyPair,
+        KeyAlgorithm, KeyInfoSigner, SignatureAlgorithm, X509CertificateError as Error,
     },
     bcder::{
         decode::Constructed,
@@ -18,7 +19,10 @@ use {
     },
     bytes::Bytes,
     chrono::{Duration, Utc},
-    ring::signature,
+    der::Document,
+    ring::signature as ringsig,
+    signature::Signer,
+    spki::{EncodePublicKey, PublicKeyDocument},
     std::{
         cmp::Ordering,
         collections::HashSet,
@@ -270,6 +274,21 @@ impl X509Certificate {
             .octet_bytes()
     }
 
+    /// Attempt to parse the public key data as [RsaPublicKey] parameters.
+    ///
+    /// Note that the raw integer value for modulus has a leading 0 byte. So its
+    /// raw length will be 1 greater than key length. e.g. an RSA 2048 key will
+    /// have `value.modulus.as_slice().len() == 257` instead of `256`.
+    pub fn rsa_public_key_data(&self) -> Result<RsaPublicKey, Error> {
+        let der = self.public_key_data();
+
+        Ok(Constructed::decode(
+            der.as_ref(),
+            Mode::Der,
+            RsaPublicKey::take_from,
+        )?)
+    }
+
     /// Compare 2 instances, sorting them so the issuer comes before the issued.
     ///
     /// This function examines the [Self::issuer_name] and [Self::subject_name]
@@ -354,6 +373,21 @@ impl AsMut<rfc5280::Certificate> for X509Certificate {
     }
 }
 
+impl EncodePublicKey for X509Certificate {
+    fn to_public_key_der(&self) -> spki::Result<PublicKeyDocument> {
+        let mut data = vec![];
+
+        self.0
+            .tbs_certificate
+            .subject_public_key_info
+            .encode_ref()
+            .write_encoded(Mode::Der, &mut data)
+            .map_err(|_| spki::Error::Asn1(der::Error::new(der::ErrorKind::Failed, 0u8.into())))?;
+
+        PublicKeyDocument::from_der(&data).map_err(spki::Error::Asn1)
+    }
+}
+
 #[derive(Clone, Eq, PartialEq)]
 enum OriginalData {
     Ber(Vec<u8>),
@@ -501,7 +535,13 @@ impl CapturedX509Certificate {
         self.verify_signed_by_public_key(public_key)
     }
 
-    /// Verify a signature over signed data perportedly signed by this certificate.
+    /// Verify a signature over signed data purportedly signed by this certificate.
+    ///
+    /// This is a wrapper to [Self::verify_signed_data_with_algorithm()] that will derive
+    /// the verification algorithm from the public key type type and the signature algorithm
+    /// indicated in this certificate. Typically these align. However, it is possible for
+    /// a signature to be produced with a different digest algorithm from that indicated
+    /// in this certificate.
     pub fn verify_signed_data(
         &self,
         signed_data: impl AsRef<[u8]>,
@@ -511,8 +551,21 @@ impl CapturedX509Certificate {
         let signature_algorithm = SignatureAlgorithm::try_from(self.signature_algorithm_oid())?;
         let verify_algorithm = signature_algorithm.resolve_verification_algorithm(key_algorithm)?;
 
-        let public_key =
-            signature::UnparsedPublicKey::new(verify_algorithm, self.public_key_data());
+        self.verify_signed_data_with_algorithm(signed_data, signature, verify_algorithm)
+    }
+
+    /// Verify a signature over signed data using an explicit verification algorithm.
+    ///
+    /// This is like [Self::verify_signed_data()] except the verification algorithm to use
+    /// is passed in instead of derived from the default algorithm for the signing key's
+    /// type.
+    pub fn verify_signed_data_with_algorithm(
+        &self,
+        signed_data: impl AsRef<[u8]>,
+        signature: impl AsRef<[u8]>,
+        verify_algorithm: &'static dyn ringsig::VerificationAlgorithm,
+    ) -> Result<(), Error> {
+        let public_key = ringsig::UnparsedPublicKey::new(verify_algorithm, self.public_key_data());
 
         public_key
             .verify(signed_data.as_ref(), signature.as_ref())
@@ -561,7 +614,7 @@ impl CapturedX509Certificate {
 
         let verify_algorithm = signature_algorithm.resolve_verification_algorithm(key_algorithm)?;
 
-        let public_key = signature::UnparsedPublicKey::new(verify_algorithm, public_key_data);
+        let public_key = ringsig::UnparsedPublicKey::new(verify_algorithm, public_key_data);
 
         public_key
             .verify(signed_data, &signature)
@@ -802,6 +855,9 @@ impl From<KeyUsage> for u8 {
 /// * There is no issuer. If no attempt is made to define an issuer,
 ///   the subject will be copied to the issuer field and this will be
 ///   a self-signed certificate.
+///
+/// This type can also be used to produce certificate signing requests. In this mode,
+/// only the subject value and additional registered attributes are meaningful.
 pub struct X509CertificateBuilder {
     key_algorithm: KeyAlgorithm,
     subject: Name,
@@ -810,6 +866,7 @@ pub struct X509CertificateBuilder {
     serial_number: i64,
     not_before: chrono::DateTime<Utc>,
     not_after: chrono::DateTime<Utc>,
+    csr_attributes: Attributes,
 }
 
 impl X509CertificateBuilder {
@@ -825,6 +882,7 @@ impl X509CertificateBuilder {
             serial_number: 1,
             not_before,
             not_after,
+            csr_attributes: Attributes::default(),
         }
     }
 
@@ -892,6 +950,14 @@ impl X509CertificateBuilder {
         });
     }
 
+    /// Add an [Attribute] to a future certificate signing requests.
+    ///
+    /// Has no effect on regular certificate creation: only if creating certificate
+    /// signing requests.
+    pub fn add_csr_attribute(&mut self, attribute: rfc5652::Attribute) {
+        self.csr_attributes.push(attribute);
+    }
+
     /// Create a new certificate given settings, using a randomly generated key pair.
     pub fn create_with_random_keypair(
         &self,
@@ -916,7 +982,7 @@ impl X509CertificateBuilder {
         let tbs_certificate = rfc5280::TbsCertificate {
             version: Some(rfc5280::Version::V3),
             serial_number: self.serial_number.into(),
-            signature: key_pair_signature_algorithm.into(),
+            signature: key_pair_signature_algorithm?.into(),
             issuer: issuer.clone(),
             validity: rfc5280::Validity {
                 not_before: Time::from(self.not_before),
@@ -924,11 +990,11 @@ impl X509CertificateBuilder {
             },
             subject: self.subject.clone(),
             subject_public_key_info: rfc5280::SubjectPublicKeyInfo {
-                algorithm: key_pair.key_algorithm().into(),
-                subject_public_key: BitString::new(
-                    0,
-                    Bytes::copy_from_slice(key_pair.public_key_data()),
-                ),
+                algorithm: key_pair
+                    .key_algorithm()
+                    .expect("InMemorySigningKeyPair always has known key algorithm")
+                    .into(),
+                subject_public_key: BitString::new(0, key_pair.public_key_data()),
             },
             issuer_unique_id: None,
             subject_unique_id: None,
@@ -947,7 +1013,8 @@ impl X509CertificateBuilder {
             .encode_ref()
             .write_encoded(Mode::Der, &mut tbs_der)?;
 
-        let (signature, signature_algorithm) = key_pair.sign(&tbs_der)?;
+        let signature = key_pair.try_sign(&tbs_der)?;
+        let signature_algorithm = key_pair.signature_algorithm()?;
 
         let cert = rfc5280::Certificate {
             tbs_certificate,
@@ -962,11 +1029,58 @@ impl X509CertificateBuilder {
 
         Ok((cert, key_pair, document))
     }
+
+    /// Create a new certificate signing request (CSR).
+    ///
+    /// The CSR is derived according to the process defined in RFC 2986 Section 3.
+    /// Essentially, we collect metadata about the request, sign that metadata using
+    /// a provided signing/private key, then attach the signature to form a complete
+    /// certification request.
+    pub fn create_certificate_signing_request(
+        &self,
+        signer: &dyn KeyInfoSigner,
+    ) -> Result<rfc2986::CertificationRequest, Error> {
+        let info = rfc2986::CertificationRequestInfo {
+            version: rfc2986::Version::V1,
+            subject: self.subject.clone(),
+            subject_public_key_info: rfc5280::SubjectPublicKeyInfo {
+                algorithm: signer
+                    .key_algorithm()
+                    .ok_or_else(|| {
+                        Error::UnknownKeyAlgorithm(
+                            "OID not available due to API limitations".into(),
+                        )
+                    })?
+                    .into(),
+                subject_public_key: BitString::new(0, signer.public_key_data()),
+            },
+            attributes: self.csr_attributes.clone(),
+        };
+
+        // The signature is produced over the DER encoding of CertificationRequestInfo
+        // per RFC 2986 Section 4.2.
+        let mut info_der = vec![];
+        info.write_encoded(Mode::Der, &mut info_der)?;
+
+        let signature = signer.try_sign(&info_der)?;
+        let signature_algorithm = signer.signature_algorithm()?;
+
+        let request = rfc2986::CertificationRequest {
+            certificate_request_info: info,
+            signature_algorithm: signature_algorithm.into(),
+            signature: BitString::new(0, signature.into()),
+        };
+
+        Ok(request)
+    }
 }
 
 #[cfg(test)]
 mod test {
-    use {super::*, crate::EcdsaCurve};
+    use {
+        super::*,
+        crate::{EcdsaCurve, X509CertificateError},
+    };
 
     #[test]
     fn builder_ed25519_default() {
@@ -999,12 +1113,36 @@ mod test {
         builder.create_with_random_keypair().unwrap();
     }
 
+    #[test]
+    fn builder_csr_ecdsa() -> Result<(), Error> {
+        for curve in EcdsaCurve::all() {
+            let key_algorithm = KeyAlgorithm::Ecdsa(*curve);
+
+            let key = InMemorySigningKeyPair::generate_random(key_algorithm)?.0;
+
+            let builder = X509CertificateBuilder::new(key_algorithm);
+
+            let csr = builder.create_certificate_signing_request(&key)?;
+
+            assert_eq!(
+                csr.certificate_request_info
+                    .subject_public_key_info
+                    .algorithm,
+                key_algorithm.into()
+            );
+        }
+
+        Ok(())
+    }
+
     #[test]
     fn ecdsa_p256_sha256_self_signed() {
         let der = include_bytes!("testdata/ecdsa-p256-sha256-self-signed.cer");
 
         let cert = CapturedX509Certificate::from_der(der.to_vec()).unwrap();
         cert.verify_signed_by_certificate(&cert).unwrap();
+
+        cert.to_public_key_der().unwrap();
     }
 
     #[test]
@@ -1013,6 +1151,7 @@ mod test {
 
         let cert = CapturedX509Certificate::from_der(der.to_vec()).unwrap();
         cert.verify_signed_by_certificate(&cert).unwrap();
+        cert.to_public_key_der().unwrap();
     }
 
     #[test]
@@ -1022,10 +1161,25 @@ mod test {
         // We can parse this. But we don't support secp512 elliptic curves because ring
         // doesn't support it.
         let cert = CapturedX509Certificate::from_der(der.to_vec()).unwrap();
+        cert.to_public_key_der().unwrap();
 
         assert!(matches!(
             cert.verify_signed_by_certificate(&cert),
             Err(Error::UnknownEllipticCurve(_))
         ));
     }
+
+    #[test]
+    fn ecdsa_prime256v1_cert_validation() -> Result<(), X509CertificateError> {
+        let root = include_bytes!("testdata/ecdsa-prime256v1-root.der");
+        let signed = include_bytes!("testdata/ecdsa-prime256v1-signed.der");
+
+        let root = CapturedX509Certificate::from_der(root.as_ref())?;
+        let signed = CapturedX509Certificate::from_der(signed.as_ref())?;
+
+        root.verify_signed_by_certificate(&root)?;
+        signed.verify_signed_by_certificate(&root)?;
+
+        Ok(())
+    }
 }
