@@ -72,7 +72,9 @@ pub mod certificate;
 pub use certificate::{
     CapturedX509Certificate, MutableX509Certificate, X509Certificate, X509CertificateBuilder,
 };
+pub mod rfc2986;
 pub mod rfc3280;
+pub mod rfc3447;
 pub mod rfc4519;
 pub mod rfc5280;
 pub mod rfc5480;
@@ -81,12 +83,14 @@ pub mod rfc5915;
 pub mod rfc5958;
 pub mod rfc8017;
 pub mod signing;
-pub use signing::InMemorySigningKeyPair;
+pub use signing::{InMemorySigningKeyPair, KeyInfoSigner, Sign, Signature};
 #[cfg(any(feature = "test", test))]
 pub mod testutil;
 
 use thiserror::Error;
 
+pub use signature::Signer;
+
 /// Errors related to X.509 certificate handling.
 #[derive(Debug, Error)]
 pub enum X509CertificateError {
@@ -120,6 +124,9 @@ pub enum X509CertificateError {
     #[error("error decoding PEM data: {0}")]
     PemDecode(pem::PemError),
 
+    #[error("error creating signature: {0}")]
+    SigningError(#[from] signature::Error),
+
     #[error("error creating cryptographic signature with memory-backed key-pair")]
     SignatureCreationInMemoryKey,
 
@@ -131,6 +138,12 @@ pub enum X509CertificateError {
 
     #[error("RSA key generation is not supported")]
     RsaKeyGenerationNotSupported,
+
+    #[error("target length for PKCS#1 padding to too short")]
+    PkcsEncodeTooShort,
+
+    #[error("unhandled error: {0}")]
+    Other(String),
 }
 
 impl From<ring::error::KeyRejected> for X509CertificateError {
