@@ -4,16 +4,90 @@
 
 use {
     crate::{
-        rfc5958::OneAsymmetricKey, EcdsaCurve, KeyAlgorithm, SignatureAlgorithm,
-        X509CertificateError as Error,
+        rfc3447::RsaPrivateKey, rfc5958::OneAsymmetricKey, EcdsaCurve, KeyAlgorithm,
+        SignatureAlgorithm, X509CertificateError as Error,
     },
     bcder::decode::Constructed,
+    bytes::Bytes,
     ring::{
         rand::SystemRandom,
-        signature::{self, KeyPair},
+        signature::{self as ringsig, KeyPair},
     },
+    signature::{Signature as SignatureTrait, Signer},
 };
 
+/// Signifies that an entity is capable of producing cryptographic signatures.
+pub trait Sign {
+    /// Create a cyrptographic signature over a message.
+    ///
+    /// Takes the message to be signed, which will be digested by the implementation.
+    ///
+    /// Returns the raw bytes constituting the signature and which signature algorithm
+    /// was used. The returned [SignatureAlgorithm] can be serialized into an
+    /// ASN.1 `AlgorithmIdentifier` via `.into()`.
+    #[deprecated(since = "0.13.0", note = "use the signature::Signer trait instead")]
+    fn sign(&self, message: &[u8]) -> Result<(Vec<u8>, SignatureAlgorithm), Error>;
+
+    /// Obtain the algorithm of the private key.
+    ///
+    /// If we can't coerce the key algorithm to [KeyAlgorithm], None is returned.
+    fn key_algorithm(&self) -> Option<KeyAlgorithm>;
+
+    /// Obtain the raw bytes constituting the public key of the signing certificate.
+    ///
+    /// This will be `.tbs_certificate.subject_public_key_info.subject_public_key` of a parsed
+    /// X.509 public certificate.
+    fn public_key_data(&self) -> Bytes;
+
+    /// Obtain the [SignatureAlgorithm] that this signer will use.
+    ///
+    /// Instances can be coerced into the ASN.1 `AlgorithmIdentifier` via `.into()`
+    /// for easy inclusion in ASN.1 structures.
+    fn signature_algorithm(&self) -> Result<SignatureAlgorithm, Error>;
+
+    /// Obtain the raw private key data.
+    fn private_key_data(&self) -> Option<Vec<u8>>;
+
+    /// Obtain RSA key primes p and q, if available.
+    fn rsa_primes(&self) -> Result<Option<(Vec<u8>, Vec<u8>)>, Error>;
+}
+
+/// A superset of [Signer] and [Sign].
+pub trait KeyInfoSigner: Signer<Signature> + Sign {}
+
+#[derive(Clone, Debug)]
+pub struct Signature(Vec<u8>);
+
+impl From<Vec<u8>> for Signature {
+    fn from(v: Vec<u8>) -> Self {
+        Self(v)
+    }
+}
+
+impl From<Signature> for Vec<u8> {
+    fn from(v: Signature) -> Vec<u8> {
+        v.0
+    }
+}
+
+impl From<Signature> for Bytes {
+    fn from(v: Signature) -> Self {
+        Self::copy_from_slice(&v.0)
+    }
+}
+
+impl AsRef<[u8]> for Signature {
+    fn as_ref(&self) -> &[u8] {
+        &self.0
+    }
+}
+
+impl SignatureTrait for Signature {
+    fn from_bytes(bytes: &[u8]) -> Result<Self, signature::Error> {
+        Ok(Self(bytes.to_vec()))
+    }
+}
+
 /// Represents a key pair that exists in memory and can be used to create cryptographic signatures.
 ///
 /// This is a wrapper around ring's various key pair types. It provides
@@ -21,15 +95,116 @@ use {
 #[derive(Debug)]
 pub enum InMemorySigningKeyPair {
     /// ECDSA key pair.
-    Ecdsa(signature::EcdsaKeyPair, EcdsaCurve),
+    Ecdsa(ringsig::EcdsaKeyPair, EcdsaCurve, Vec<u8>),
 
     /// ED25519 key pair.
-    Ed25519(signature::Ed25519KeyPair),
+    Ed25519(ringsig::Ed25519KeyPair),
 
     /// RSA key pair.
-    Rsa(signature::RsaKeyPair),
+    Rsa(ringsig::RsaKeyPair, Vec<u8>),
 }
 
+impl Signer<Signature> for InMemorySigningKeyPair {
+    fn try_sign(&self, msg: &[u8]) -> Result<Signature, signature::Error> {
+        match self {
+            Self::Rsa(key, _) => {
+                let mut signature = vec![0; key.public_modulus_len()];
+
+                key.sign(
+                    &ringsig::RSA_PKCS1_SHA256,
+                    &ring::rand::SystemRandom::new(),
+                    msg,
+                    &mut signature,
+                )
+                .map_err(|_| signature::Error::new())?;
+
+                Ok(signature.into())
+            }
+            Self::Ecdsa(key, _, _) => {
+                let signature = key
+                    .sign(&ring::rand::SystemRandom::new(), msg)
+                    .map_err(|_| signature::Error::new())?;
+
+                Signature::from_bytes(signature.as_ref())
+            }
+            Self::Ed25519(key) => {
+                let signature = key.sign(msg);
+
+                Signature::from_bytes(signature.as_ref())
+            }
+        }
+    }
+}
+
+impl Sign for InMemorySigningKeyPair {
+    /// This will use a new instance of ring's SystemRandom. The RSA
+    /// padding algorithm is hard-coded to RSA_PCS1_SHA256.
+    ///
+    /// If you want total control over signing parameters, obtain the
+    /// underlying ring keypair and call its `.sign()`.
+    fn sign(&self, message: &[u8]) -> Result<(Vec<u8>, SignatureAlgorithm), Error> {
+        let algorithm = self.signature_algorithm()?;
+
+        Ok((self.try_sign(message)?.into(), algorithm))
+    }
+
+    fn key_algorithm(&self) -> Option<KeyAlgorithm> {
+        Some(match self {
+            Self::Rsa(_, _) => KeyAlgorithm::Rsa,
+            Self::Ed25519(_) => KeyAlgorithm::Ed25519,
+            Self::Ecdsa(_, curve, _) => KeyAlgorithm::Ecdsa(*curve),
+        })
+    }
+
+    fn public_key_data(&self) -> Bytes {
+        match self {
+            Self::Rsa(key, _) => Bytes::copy_from_slice(key.public_key().as_ref()),
+            Self::Ecdsa(key, _, _) => Bytes::copy_from_slice(key.public_key().as_ref()),
+            Self::Ed25519(key) => Bytes::copy_from_slice(key.public_key().as_ref()),
+        }
+    }
+
+    fn signature_algorithm(&self) -> Result<SignatureAlgorithm, Error> {
+        Ok(match self {
+            Self::Rsa(_, _) => SignatureAlgorithm::RsaSha256,
+            Self::Ecdsa(_, curve, _) => {
+                // ring refuses to mix and match the bitness of curves and signature
+                // algorithms. e.g. it can't pair secp256r1 with SHA-384. It chooses
+                // signatures on its own. We reimplement that logic here.
+                match curve {
+                    EcdsaCurve::Secp256r1 => SignatureAlgorithm::EcdsaSha256,
+                    EcdsaCurve::Secp384r1 => SignatureAlgorithm::EcdsaSha384,
+                }
+            }
+            Self::Ed25519(_) => SignatureAlgorithm::Ed25519,
+        })
+    }
+
+    fn private_key_data(&self) -> Option<Vec<u8>> {
+        match self {
+            Self::Rsa(_, data) => Some(data.clone()),
+            Self::Ecdsa(_, _, data) => Some(data.clone()),
+            Self::Ed25519(_) => None,
+        }
+    }
+
+    fn rsa_primes(&self) -> Result<Option<(Vec<u8>, Vec<u8>)>, Error> {
+        match self {
+            Self::Rsa(_, data) => {
+                let key = Constructed::decode(data.as_ref(), bcder::Mode::Der, |cons| {
+                    RsaPrivateKey::take_from(cons)
+                })?;
+
+                Ok(Some((key.p.as_slice().to_vec(), key.q.as_slice().to_vec())))
+            }
+            Self::Ecdsa(..) => Ok(None),
+            Self::Ed25519(_) => Ok(None),
+        }
+    }
+}
+
+impl KeyInfoSigner for InMemorySigningKeyPair {}
+
 impl InMemorySigningKeyPair {
     /// Attempt to instantiate an instance from PKCS#8 DER data.
     ///
@@ -45,12 +220,17 @@ impl InMemorySigningKeyPair {
         // self.key_algorithm() assumes a 1:1 mapping between KeyAlgorithm and our enum
         // variants. If you change this, change that function as well.
         match algorithm {
-            KeyAlgorithm::Rsa => Ok(Self::Rsa(signature::RsaKeyPair::from_pkcs8(data.as_ref())?)),
-            KeyAlgorithm::Ecdsa(curve) => Ok(Self::Ecdsa(
-                signature::EcdsaKeyPair::from_pkcs8(curve.into(), data.as_ref())?,
-                curve,
-            )),
-            KeyAlgorithm::Ed25519 => Ok(Self::Ed25519(signature::Ed25519KeyPair::from_pkcs8(
+            KeyAlgorithm::Rsa => {
+                let pair = ringsig::RsaKeyPair::from_pkcs8(data.as_ref())?;
+
+                Ok(Self::Rsa(pair, key.private_key.into_bytes().to_vec()))
+            }
+            KeyAlgorithm::Ecdsa(curve) => {
+                let pair = ringsig::EcdsaKeyPair::from_pkcs8(curve.into(), data.as_ref())?;
+
+                Ok(Self::Ecdsa(pair, curve, data.as_ref().to_vec()))
+            }
+            KeyAlgorithm::Ed25519 => Ok(Self::Ed25519(ringsig::Ed25519KeyPair::from_pkcs8(
                 data.as_ref(),
             )?)),
         }
@@ -77,12 +257,10 @@ impl InMemorySigningKeyPair {
         let rng = SystemRandom::new();
 
         let document = match key_algorithm {
-            KeyAlgorithm::Ed25519 => signature::Ed25519KeyPair::generate_pkcs8(&rng)
+            KeyAlgorithm::Ed25519 => ringsig::Ed25519KeyPair::generate_pkcs8(&rng)
+                .map_err(|_| Error::KeyPairGenerationError),
+            KeyAlgorithm::Ecdsa(curve) => ringsig::EcdsaKeyPair::generate_pkcs8(curve.into(), &rng)
                 .map_err(|_| Error::KeyPairGenerationError),
-            KeyAlgorithm::Ecdsa(curve) => {
-                signature::EcdsaKeyPair::generate_pkcs8(curve.into(), &rng)
-                    .map_err(|_| Error::KeyPairGenerationError)
-            }
             KeyAlgorithm::Rsa => Err(Error::RsaKeyGenerationNotSupported),
         }?;
 
@@ -91,119 +269,36 @@ impl InMemorySigningKeyPair {
         Ok((key_pair, document))
     }
 
-    /// Obtain the raw bytes constituting the key pair's public key.
-    pub fn public_key_data(&self) -> &[u8] {
-        match self {
-            Self::Rsa(key) => key.public_key().as_ref(),
-            Self::Ecdsa(key, _) => key.public_key().as_ref(),
-            Self::Ed25519(key) => key.public_key().as_ref(),
-        }
-    }
-
-    /// Obtain the [KeyAlgorithm] in use by this instance.
-    pub fn key_algorithm(&self) -> KeyAlgorithm {
-        match self {
-            Self::Rsa(_) => KeyAlgorithm::Rsa,
-            Self::Ed25519(_) => KeyAlgorithm::Ed25519,
-            Self::Ecdsa(_, curve) => KeyAlgorithm::Ecdsa(*curve),
-        }
-    }
-
-    /// Obtain the [SignatureAlgorithm] that this key pair will use.
-    ///
-    /// Instances can be coerced into the ASN.1 `AlgorithmIdentifier` via `.into()`
-    /// for easy inclusion in ASN.1 structures.
-    pub fn signature_algorithm(&self) -> SignatureAlgorithm {
-        match self {
-            Self::Rsa(_) => SignatureAlgorithm::RsaSha256,
-            Self::Ecdsa(_, curve) => {
-                // ring refuses to mix and match the bitness of curves and signature
-                // algorithms. e.g. it can't pair secp256r1 with SHA-384. It chooses
-                // signatures on its own. We reimplement that logic here.
-                match curve {
-                    EcdsaCurve::Secp256r1 => SignatureAlgorithm::EcdsaSha256,
-                    EcdsaCurve::Secp384r1 => SignatureAlgorithm::EcdsaSha384,
-                }
-            }
-            Self::Ed25519(_) => SignatureAlgorithm::Ed25519,
-        }
-    }
-
     /// Attempt to resolve a verification algorithm for this key pair.
     ///
     /// This is a wrapper around [SignatureAlgorithm::resolve_verification_algorithm()]
     /// with our bound [KeyAlgorithm]. However, since there are no parameters
     /// that can result in wrong choices, this is guaranteed to always work
     /// and doesn't require `Result`.
-    pub fn verification_algorithm(&self) -> &'static dyn signature::VerificationAlgorithm {
-        self.signature_algorithm()
-            .resolve_verification_algorithm(self.key_algorithm()).expect(
+    pub fn verification_algorithm(
+        &self,
+    ) -> Result<&'static dyn ringsig::VerificationAlgorithm, Error> {
+        Ok(self.signature_algorithm()?
+            .resolve_verification_algorithm(self.key_algorithm().expect("key algorithm should be known for InMemorySigningKeyPair")).expect(
             "illegal combination of key algorithm in signature algorithm: this should not occur"
-        )
-    }
-
-    /// Sign a message using this signing key.
-    ///
-    /// Returns the raw bytes constituting the signature and which signature
-    /// algorithm was used. The returned [SignatureAlgorithm] can be serialized
-    /// into an ASN.1 `AlgorithmIdentifier` via `.into()`.
-    ///
-    /// This will use a new instance of ring's SystemRandom. The RSA
-    /// padding algorithm is hard-coded to RSA_PCS1_SHA256.
-    ///
-    /// If you want total control over signing parameters, obtain the
-    /// underlying ring keypair and call its `.sign()`.
-    pub fn sign(&self, message: impl AsRef<[u8]>) -> Result<(Vec<u8>, SignatureAlgorithm), Error> {
-        match self {
-            Self::Rsa(key) => {
-                let mut signature = vec![0; key.public_modulus_len()];
-
-                key.sign(
-                    &ring::signature::RSA_PKCS1_SHA256,
-                    &ring::rand::SystemRandom::new(),
-                    message.as_ref(),
-                    &mut signature,
-                )
-                .map_err(|_| Error::SignatureCreationInMemoryKey)?;
-
-                Ok((signature, self.signature_algorithm()))
-            }
-            Self::Ecdsa(key, _) => {
-                let signature = key
-                    .sign(&ring::rand::SystemRandom::new(), message.as_ref())
-                    .map_err(|_| Error::SignatureCreationInMemoryKey)?;
-
-                Ok((signature.as_ref().to_vec(), self.signature_algorithm()))
-            }
-            Self::Ed25519(key) => {
-                let signature = key.sign(message.as_ref());
-
-                Ok((signature.as_ref().to_vec(), self.signature_algorithm()))
-            }
-        }
+        ))
     }
 }
 
 // We don't support coercing from EcdsaKeyPair because we don't know what the
 // elliptic curve is since ring doesn't expose it.
 
-impl From<signature::Ed25519KeyPair> for InMemorySigningKeyPair {
-    fn from(key: signature::Ed25519KeyPair) -> Self {
+impl From<ringsig::Ed25519KeyPair> for InMemorySigningKeyPair {
+    fn from(key: ringsig::Ed25519KeyPair) -> Self {
         Self::Ed25519(key)
     }
 }
 
-impl From<signature::RsaKeyPair> for InMemorySigningKeyPair {
-    fn from(key: signature::RsaKeyPair) -> Self {
-        Self::Rsa(key)
-    }
-}
-
 impl From<&InMemorySigningKeyPair> for KeyAlgorithm {
     fn from(key: &InMemorySigningKeyPair) -> Self {
         match key {
-            InMemorySigningKeyPair::Rsa(_) => KeyAlgorithm::Rsa,
-            InMemorySigningKeyPair::Ecdsa(_, curve) => KeyAlgorithm::Ecdsa(*curve),
+            InMemorySigningKeyPair::Rsa(_, _) => KeyAlgorithm::Rsa,
+            InMemorySigningKeyPair::Ecdsa(_, curve, _) => KeyAlgorithm::Ecdsa(*curve),
             InMemorySigningKeyPair::Ed25519(_) => KeyAlgorithm::Ed25519,
         }
     }
@@ -211,7 +306,7 @@ impl From<&InMemorySigningKeyPair> for KeyAlgorithm {
 
 #[cfg(test)]
 mod test {
-    use {super::*, crate::rfc5280, crate::testutil::*, ring::signature::UnparsedPublicKey};
+    use {super::*, crate::rfc5280, crate::testutil::*, ringsig::UnparsedPublicKey};
 
     #[test]
     fn generate_random_ecdsa() {
@@ -235,13 +330,16 @@ mod test {
         let rng = ring::rand::SystemRandom::new();
 
         for alg in &[
-            &signature::ECDSA_P256_SHA256_ASN1_SIGNING,
-            &signature::ECDSA_P384_SHA384_ASN1_SIGNING,
+            &ringsig::ECDSA_P256_SHA256_ASN1_SIGNING,
+            &ringsig::ECDSA_P384_SHA384_ASN1_SIGNING,
         ] {
-            let doc = ring::signature::EcdsaKeyPair::generate_pkcs8(alg, &rng).unwrap();
+            let doc = ringsig::EcdsaKeyPair::generate_pkcs8(alg, &rng).unwrap();
 
             let signing_key = InMemorySigningKeyPair::from_pkcs8_der(doc.as_ref()).unwrap();
-            assert!(matches!(signing_key, InMemorySigningKeyPair::Ecdsa(_, _)));
+            assert!(matches!(
+                signing_key,
+                InMemorySigningKeyPair::Ecdsa(_, _, _)
+            ));
 
             let pem_data = pem::encode(&pem::Pem {
                 tag: "PRIVATE KEY".to_string(),
@@ -249,7 +347,10 @@ mod test {
             });
 
             let signing_key = InMemorySigningKeyPair::from_pkcs8_pem(pem_data.as_bytes()).unwrap();
-            assert!(matches!(signing_key, InMemorySigningKeyPair::Ecdsa(_, _)));
+            assert!(matches!(
+                signing_key,
+                InMemorySigningKeyPair::Ecdsa(_, _, _)
+            ));
 
             let key_pair_asn1 = Constructed::decode(doc.as_ref(), bcder::Mode::Der, |cons| {
                 OneAsymmetricKey::take_from(cons)
@@ -261,9 +362,9 @@ mod test {
                 KeyAlgorithm::Ecdsa(EcdsaCurve::Secp256r1).into()
             );
 
-            let expected = if *alg == &signature::ECDSA_P256_SHA256_ASN1_SIGNING {
+            let expected = if *alg == &ringsig::ECDSA_P256_SHA256_ASN1_SIGNING {
                 EcdsaCurve::Secp256r1
-            } else if *alg == &signature::ECDSA_P384_SHA384_ASN1_SIGNING {
+            } else if *alg == &ringsig::ECDSA_P384_SHA384_ASN1_SIGNING {
                 EcdsaCurve::Secp384r1
             } else {
                 panic!("unhandled test case");
@@ -285,7 +386,7 @@ mod test {
     fn signing_key_from_ed25519_pkcs8() {
         let rng = ring::rand::SystemRandom::new();
 
-        let doc = ring::signature::Ed25519KeyPair::generate_pkcs8(&rng).unwrap();
+        let doc = ringsig::Ed25519KeyPair::generate_pkcs8(&rng).unwrap();
 
         let signing_key = InMemorySigningKeyPair::from_pkcs8_der(doc.as_ref()).unwrap();
         assert!(matches!(signing_key, InMemorySigningKeyPair::Ed25519(_)));
@@ -366,11 +467,13 @@ mod test {
         let cert = rsa_cert();
         let message = b"hello, world";
 
-        let (signature, _) = key.sign(message).unwrap();
+        let signature = Signer::try_sign(&key, message).unwrap();
 
-        let public_key =
-            UnparsedPublicKey::new(key.verification_algorithm(), cert.public_key_data());
+        let public_key = UnparsedPublicKey::new(
+            key.verification_algorithm().unwrap(),
+            cert.public_key_data(),
+        );
 
-        public_key.verify(message, &signature).unwrap();
+        public_key.verify(message, signature.as_ref()).unwrap();
     }
 }
